@@ -6,7 +6,7 @@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
-* the Free Software Foundation, either version 3 of the License, or
+* the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
@@ -50,8 +50,10 @@ int QObject_Finalize( struct VMGlobals *, struct PyrObject * );
 
 QC_LANG_PRIMITIVE( QObject_New, 2, PyrSlot *r, PyrSlot *a, VMGlobals *g )
 {
+  if(NotSym(a+0)) return errWrongType;
+
   PyrObject *scObject = slotRawObject( r );
-  QString qtClassName = Slot::toString( a+0 );
+  QString qtClassName = QString(slotRawSymbol(a+0)->name);
 
   qcSCObjectDebugMsg( 1, scObject, QString("CREATE: %2").arg(qtClassName) );
 
@@ -70,7 +72,7 @@ QC_LANG_PRIMITIVE( QObject_New, 2, PyrSlot *r, PyrSlot *a, VMGlobals *g )
 
   PyrSlot *slotArg = a+1;
 
-  if( isKindOfSlot( slotArg, class_Array ) ) {
+  if( isKindOfSlot( slotArg, class_array ) ) {
     PyrObject *array = slotRawObject( slotArg );
     PyrSlot *s = array->slots;
     int size = array->size;
@@ -124,17 +126,6 @@ int QObject_Finalize( struct VMGlobals *, struct PyrObject *obj )
   return errNone;
 }
 
-QC_LANG_PRIMITIVE( QObject_ManuallyFinalize, 0, PyrSlot *r, PyrSlot *a, VMGlobals *g )
-{
-  qcSCObjectDebugMsg( 1, slotRawObject(r), "MANUAL FINALIZE" );
-
-  // WARNING we assume that proxy's deletion will be deferred until any
-  // language shutdown code using it will have been executed, so any
-  // shutdown code is safe.
-
-  return QObject_Finalize( g, slotRawObject(r) );
-}
-
 QC_LANG_PRIMITIVE( QObject_SetParent, 1, PyrSlot *r, PyrSlot *a, VMGlobals *g )
 {
   QObjectProxy *proxy = QOBJECT_FROM_SLOT( r );
@@ -202,7 +193,9 @@ QC_LANG_PRIMITIVE( QMetaObject_Properties, 0, PyrSlot *r, PyrSlot *a, VMGlobals
 {
   if( !QcApplication::compareThread() ) return QtCollider::wrongThreadError();
 
-  QString className = Slot::toString( slotRawObject(r)->slots+0 );
+  PyrSlot *sClassName = slotRawObject(r)->slots+0;
+  if(NotSym(sClassName)) return errWrongType;
+  QString className( slotRawSymbol(sClassName)->name );
 
   QcAbstractFactory *f = QtCollider::factories().value( className );
 
@@ -221,7 +214,9 @@ QC_LANG_PRIMITIVE( QMetaObject_Methods, 3, PyrSlot *r, PyrSlot *a, VMGlobals *g
 {
   if( !QcApplication::compareThread() ) return QtCollider::wrongThreadError();
 
-  QString className = Slot::toString(slotRawObject(r)->slots+0);
+  PyrSlot *sClassName = slotRawObject(r)->slots+0;
+  if(NotSym(sClassName)) return errWrongType;
+  QString className( slotRawSymbol(sClassName)->name );
 
   QcAbstractFactory *f = QtCollider::factories().value( className );
 
@@ -373,54 +368,57 @@ QC_LANG_PRIMITIVE( QObject_SetEventHandlerEnabled, 2, PyrSlot *r, PyrSlot *a, VM
 
 QC_LANG_PRIMITIVE( QObject_ConnectMethod, 3, PyrSlot *r, PyrSlot *a, VMGlobals *g )
 {
-  QString signal = Slot::toString( a+0 );
-  if( signal.isEmpty() || NotSym( a+1 ) ) return errWrongType;
-  PyrSymbol *handler = 0; slotSymbolVal( a+1, &handler );
+  if( NotSym(a+0) || NotSym( a+1 ) ) return errWrongType;
+
+  PyrSymbol *signal = slotRawSymbol(a+0);
+  PyrSymbol *handler = slotRawSymbol(a+1);
   Qt::ConnectionType ctype =
     Slot::toBool( a+2 ) ? Qt::DirectConnection : Qt::QueuedConnection;
 
   qcSCObjectDebugMsg( 1, slotRawObject(r),
-                      QString("CONNECT METHOD: %1 -> %2 [%3]").arg(signal).arg(handler->name)
+                      QString("CONNECT METHOD: %1 -> %2 [%3]").arg(signal->name).arg(handler->name)
                       .arg( IsTrue(a+2) ? "SYNC" : "ASYNC") );
 
   QObjectProxy *proxy = QOBJECT_FROM_SLOT( r );
 
   if( !proxy->compareThread() ) return QtCollider::wrongThreadError();
 
-  bool ok = proxy->connectMethod( signal.toAscii().constData(), handler, ctype );
+  bool ok = proxy->connectMethod( signal->name, handler, ctype );
 
   return ok ? errNone : errFailed;
 }
 
 QC_LANG_PRIMITIVE( QObject_DisconnectMethod, 2, PyrSlot *r, PyrSlot *a, VMGlobals *g )
 {
-  QString signal = Slot::toString( a+0 );
-  if( signal.isEmpty() || NotSym( a+1 ) ) return errWrongType;
-  PyrSymbol *handler = 0; slotSymbolVal( a+1, &handler );
+  if( NotSym(a+0) || NotSym( a+1 ) ) return errWrongType;
+
+  PyrSymbol *signal = slotRawSymbol(a+0);
+  PyrSymbol *handler = slotRawSymbol(a+1);
 
   qcSCObjectDebugMsg( 1, slotRawObject(r),
-                      QString("DISCONNECT METHOD: %1 -> %2").arg(signal).arg(handler->name) );
+                      QString("DISCONNECT METHOD: %1 -> %2").arg(signal->name).arg(handler->name) );
 
   QObjectProxy *proxy = QOBJECT_FROM_SLOT( r );
 
   if( !proxy->compareThread() ) return QtCollider::wrongThreadError();
 
-  bool ok = proxy->disconnectMethod( signal.toAscii().constData(), handler );
+  bool ok = proxy->disconnectMethod( signal->name, handler );
 
   return ok ? errNone : errFailed;
 }
 
 QC_LANG_PRIMITIVE( QObject_ConnectObject, 3, PyrSlot *r, PyrSlot *a, VMGlobals *g )
 {
-  QString signal = Slot::toString( a+0 );
-  if( signal.isEmpty() || NotObj( a+1 ) ) return errWrongType;
+  if( NotSym(a+0) || NotObj( a+1 ) ) return errWrongType;
+
+  PyrSymbol *signal = slotRawSymbol(a+0);
   PyrObject *handlerObj = slotRawObject( a+1 );
   Qt::ConnectionType ctype =
     Slot::toBool( a+2 ) ? Qt::DirectConnection : Qt::QueuedConnection;
 
   qcSCObjectDebugMsg( 1, slotRawObject(r),
                       QString("CONNECT OBJECT: %1 -> %2 [%3]")
-                      .arg(signal)
+                      .arg( signal->name )
                       .arg( slotRawSymbol( &handlerObj->classptr->name )->name )
                       .arg( IsTrue(a+2) ? "SYNC" : "ASYNC") );
 
@@ -428,25 +426,26 @@ QC_LANG_PRIMITIVE( QObject_ConnectObject, 3, PyrSlot *r, PyrSlot *a, VMGlobals *
 
   if( !proxy->compareThread() ) return QtCollider::wrongThreadError();
 
-  bool ok = proxy->connectObject( signal.toAscii().constData(), handlerObj, ctype );
+  bool ok = proxy->connectObject( signal->name, handlerObj, ctype );
 
   return ok ? errNone : errFailed;
 }
 
 QC_LANG_PRIMITIVE( QObject_DisconnectObject, 2, PyrSlot *r, PyrSlot *a, VMGlobals *g )
 {
-  QString signal = Slot::toString( a+0 );
-  if( signal.isEmpty() || NotObj( a+1 ) ) return errWrongType;
+  if( NotSym(a+0) || NotObj( a+1 ) ) return errWrongType;
+
+  PyrSymbol *signal = slotRawSymbol(a+0);
   PyrObject *handlerObj = slotRawObject( a+1 );
 
   qcSCObjectDebugMsg( 1, slotRawObject(r),
-                      QString("DISCONNECT OBJECT: %1").arg(signal) );
+                      QString("DISCONNECT OBJECT: %1").arg(signal->name) );
 
   QObjectProxy *proxy = QOBJECT_FROM_SLOT( r );
 
   if( !proxy->compareThread() ) return QtCollider::wrongThreadError();
 
-  bool ok = proxy->disconnectObject( signal.toAscii().constData(), handlerObj );
+  bool ok = proxy->disconnectObject( signal->name, handlerObj );
 
   return ok ? errNone : errFailed;
 }
@@ -454,7 +453,7 @@ QC_LANG_PRIMITIVE( QObject_DisconnectObject, 2, PyrSlot *r, PyrSlot *a, VMGlobal
 QC_LANG_PRIMITIVE( QObject_ConnectSlot, 3, PyrSlot *r, PyrSlot *a, VMGlobals *g )
 {
   // Args: signal, receiver, slot
-  if( !isKindOfSlot( a+1, class_QObject )
+  if( !isKindOfSlot( a+1, SC_CLASS(QObject) )
       || NotSym( a+0 ) || NotSym( a+2 ) ) return errWrongType;
 
   PyrSymbol *symSig = slotRawSymbol( a+0 );
@@ -525,15 +524,19 @@ QC_LANG_PRIMITIVE( QObject_InvokeMethod, 3, PyrSlot *r, PyrSlot *a, VMGlobals *g
 
 QC_LANG_PRIMITIVE( QObject_IsValid, 0, PyrSlot *r, PyrSlot *a, VMGlobals *g )
 {
-  // NOTE Though we could check validity without posting an event (by locking the proxy),
-  // we post a sync event to maximize the chance that a DeferredDelete event issued earlier
-  // can get processed by the object in question before it is checked for validity.
-
   QObjectProxy *proxy = Slot::toObjectProxy( r );
 
-  if( !proxy->compareThread() ) return QtCollider::wrongThreadError();
+  bool needLock = !proxy->compareThread();
 
-  SetBool( r, proxy->object() != 0 );
+  if (needLock)
+      proxy->lock();
+
+  bool hasObject = proxy->object() != 0;
+
+  if (needLock)
+      proxy->unlock();
+
+  SetBool( r, hasObject );
 
   return errNone;
 }
@@ -592,7 +595,6 @@ void defineQObjectPrimitives()
   LangPrimitiveDefiner definer;
   definer.define<QObject_New>();
   definer.define<QObject_Destroy>();
-  definer.define<QObject_ManuallyFinalize>();
   definer.define<QObject_SetParent>();
   definer.define<QMetaObject_Properties>();
   definer.define<QMetaObject_Methods>();
