@@ -8,7 +8,7 @@
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
-* the Free Software Foundation, either version 3 of the License, or
+* the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
@@ -24,6 +24,7 @@
 #include "primitives.h"
 #include "../painting.h"
 #include "../Slot.h"
+#include "PyrKernel.h"
 
 #include <QPainter>
 #include <QVector2D>
@@ -215,7 +216,7 @@ QC_QPEN_PRIMITIVE( QPen_SetDashPattern, 1, PyrSlot *r, PyrSlot *a, VMGlobals *g
 {
   if( !IsObj( a ) ) return errWrongType;
   PyrObject *obj = slotRawObject( a );
-  if( obj->classptr != class_FloatArray ) return errWrongType;
+  if( obj->classptr != class_floatarray ) return errWrongType;
   PyrFloatArray *farray = reinterpret_cast<PyrFloatArray*>(obj);
 
   int s = farray->size;
@@ -307,7 +308,7 @@ QC_QPEN_PRIMITIVE( QPen_Rotate, 3, PyrSlot *r, PyrSlot *a, VMGlobals *g )
   return errNone;
 }
 
-QC_QPEN_PRIMITIVE( QPen_Transform, 1, PyrSlot *r, PyrSlot *a, VMGlobals *g )
+QC_QPEN_PRIMITIVE( QPen_SetTransform, 1, PyrSlot *r, PyrSlot *a, VMGlobals *g )
 {
   VariantList list = Slot::toVariantList( a );
   if( list.data.count() < 6 ) return errWrongType;
@@ -320,10 +321,29 @@ QC_QPEN_PRIMITIVE( QPen_Transform, 1, PyrSlot *r, PyrSlot *a, VMGlobals *g )
     f[i] = list.data[i].value<float>();
   }
   QTransform transform( f[0], f[1], f[2], f[3], f[4], f[5] );
-  painter->setWorldTransform( transform );
+  painter->setWorldTransform( transform, true );
   return errNone;
 }
 
+QC_QPEN_PRIMITIVE( QPen_Transform, 0, PyrSlot *r, PyrSlot *a, VMGlobals *g )
+{
+    const QTransform & transform = painter->worldTransform();
+
+    PyrDoubleArray *doubleArray = newPyrDoubleArray( g->gc, 6, 0, true );
+    SetObject( r, doubleArray );
+
+    double *val = doubleArray->d;
+    val[0] = transform.m11();
+    val[1] = transform.m12();
+    val[2] = transform.m21();
+    val[3] = transform.m22();
+    val[4] = transform.dx();
+    val[5] = transform.dy();
+    doubleArray->size = 6;
+
+    return errNone;
+}
+
 QC_QPEN_PRIMITIVE( QPen_MoveTo, 1, PyrSlot *r, PyrSlot *a, VMGlobals *g )
 {
   QPointF point = Slot::toPoint( a );
@@ -506,6 +526,11 @@ QC_QPEN_PRIMITIVE( QPen_Draw, 1, PyrSlot *r, PyrSlot *a, VMGlobals *g )
     case 4:
     default: ;
   }
+  if ( style == 1 || style == 4 ) {
+    path.setFillRule( Qt::OddEvenFill );
+  } else {
+    path.setFillRule( Qt::WindingFill );
+  }
 
   painter->drawPath( path );
 
@@ -639,6 +664,7 @@ void defineQPenPrimitives()
   definer.define<QPen_Shear>();
   definer.define<QPen_Rotate>();
   definer.define<QPen_Transform>();
+  definer.define<QPen_SetTransform>();
   definer.define<QPen_MoveTo>();
   definer.define<QPen_LineTo>();
   definer.define<QPen_CubicTo>();
