@@ -11,9 +11,11 @@ bcp --boost=${BOOST_ROOT} \
 	--scan common/*pp \
 	--scan lang/*/*pp \
 	--scan server/supernova/*/*pp \
-	--scan external_libraries/boost_*/*/*pp \
-	--scan external_libraries/boost_*/*/*/*pp \
-	--scan external_libraries/boost_*/*/*/*/*pp \
+	--scan editors/sc-ide/*/*pp \
+	--scan editors/sc-ide/*/*/*pp \
+	--scan external_libraries/boost*/*/*pp \
+	--scan external_libraries/boost*/*/*/*pp \
+	--scan external_libraries/boost*/*/*/*/*pp \
 	--scan external_libraries/threadpool/boost/*/*pp \
 	--scan external_libraries/threadpool/boost/*/*/*pp \
 	--scan testsuite/supernova/*pp \
