@@ -29,15 +29,18 @@ Primitives for File i/o.
 #include "PyrPrimitive.h"
 #include "PyrSymbol.h"
 #include "PyrFilePrim.h"
-#include "PyrFileUtils.h"
 #include "ReadWriteMacros.h"
 #include "SCBase.h"
 #include "SC_DirUtils.h"
+#include "sc_popen.h"
+
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
 #include <cerrno>
 
+#include "../../common/SC_SndFileHelpers.hpp"
+
 #ifdef NOCLASSIC
 #include <TextUtils.h>
 #include <Navigation.h>
@@ -48,7 +51,6 @@ Primitives for File i/o.
 #else
 # include <direct.h>
 # include <malloc.h>
-# define strcasecmp stricmp
 #endif
 
 #ifdef _WIN32
@@ -75,8 +77,6 @@ Primitives for File i/o.
 
 #define DELIMITOR ':'
 
-bool filelen(FILE *file, size_t *length);
-
 int prFileDelete(struct VMGlobals *g, int numArgsPushed)
 {
 	PyrSlot *a = g->sp - 1, *b = g->sp;
@@ -161,7 +161,7 @@ int prFileRealPath(struct VMGlobals* g, int numArgsPushed )
 
 int prFileMkDir(struct VMGlobals * g, int numArgsPushed)
 {
-	PyrSlot *a = g->sp - 1, *b = g->sp;
+	PyrSlot *b = g->sp;
 	char filename[PATH_MAX];
 
 	int error = slotStrVal(b, filename, PATH_MAX);
@@ -178,7 +178,7 @@ int prFileMkDir(struct VMGlobals * g, int numArgsPushed)
 
 int prFileCopy(struct VMGlobals * g, int numArgsPushed)
 {
-	PyrSlot *a = g->sp - 2, *b = g->sp - 1, *c = g->sp;
+	PyrSlot *b = g->sp - 1, *c = g->sp;
 	char filename1[PATH_MAX];
 	char filename2[PATH_MAX];
 	int error;
@@ -189,7 +189,7 @@ int prFileCopy(struct VMGlobals * g, int numArgsPushed)
 	if (error != errNone)
 		return error;
 
-	boost::filesystem3::copy(filename1, filename2);
+	boost::filesystem::copy(filename1, filename2);
 	return errNone;
 }
 
@@ -332,7 +332,7 @@ int prFilePos(struct VMGlobals *g, int numArgsPushed)
 	if (file == NULL) return errFailed;
 	if (fgetpos(file, &pos)) return errFailed;
 
-#ifdef SC_LINUX
+#ifdef __linux__
 	// sk: hack alert!
 	length = pos.__pos;
 #else
@@ -345,19 +345,18 @@ int prFilePos(struct VMGlobals *g, int numArgsPushed)
 
 int prFileLength(struct VMGlobals *g, int numArgsPushed)
 {
-	PyrSlot *a;
-	PyrFile *pfile;
-	FILE *file;
-	fpos_t pos;
-	size_t length;
+	PyrSlot * a = g->sp;
+	PyrFile *pfile = (PyrFile*)slotRawObject(a);
+	FILE *file = (FILE*)slotRawPtr(&pfile->fileptr);
 
-	a = g->sp;
-	pfile = (PyrFile*)slotRawObject(a);
-	file = (FILE*)slotRawPtr(&pfile->fileptr);
 	if (file == NULL) return errFailed;
+
 	// preserve file position
+	fpos_t pos;
 	if (fgetpos(file, &pos)) return errFailed;
-	if (filelen(file, &length)) return errFailed;
+	if (fseek(file, 0, SEEK_END)) return errFailed;
+	size_t length;
+	length = ftell(file);
 	if (fsetpos(file, &pos)) return errFailed;
 
 	SetInt(a, length);
@@ -632,9 +631,9 @@ int prFilePutInt32(struct VMGlobals *g, int numArgsPushed)
 	file = (FILE*)slotRawPtr(&pfile->fileptr);
 	if (file == NULL) return errFailed;
 
-        int val;
-        int err = slotIntVal(b, &val);
-        if (err) return err;
+	int val;
+	int err = slotIntVal(b, &val);
+	if (err) return err;
 
 	SC_IOStream<FILE*> scio(file);
 	scio.writeInt32_be(val);
@@ -680,9 +679,9 @@ int prFilePutInt32LE(struct VMGlobals *g, int numArgsPushed)
 	file = (FILE*)slotRawPtr(&pfile->fileptr);
 	if (file == NULL) return errFailed;
 
-        int val;
-        int err = slotIntVal(b, &val);
-        if (err) return err;
+	int val;
+	int err = slotIntVal(b, &val);
+	if (err) return err;
 
 	SC_IOStream<FILE*> scio(file);
 	scio.writeInt32_le(val);
@@ -1265,7 +1264,6 @@ int prFileGetcwd(struct VMGlobals *g, int numArgsPushed)
 
 ////////
 
-#ifndef SC_WIN32
 int prPipeOpen(struct VMGlobals *g, int numArgsPushed)
 {
 	PyrSlot *a, *b, *c;
@@ -1290,14 +1288,14 @@ int prPipeOpen(struct VMGlobals *g, int numArgsPushed)
 	memcpy(mode, slotRawString(c)->s, slotRawObject(c)->size);
 	mode[slotRawString(c)->size] = 0;
 
-	file = popen(commandLine, mode);
+	pid_t pid;
+	file = sc_popen(commandLine, &pid, mode);
 	free(commandLine);
 	if (file) {
 		SetPtr(&pfile->fileptr, file);
-		SetTrue(a);
+		SetInt(a, pid);
 	} else {
 		SetNil(a);
-		SetFalse(a);
 	}
 	return errNone;
 }
@@ -1305,21 +1303,25 @@ int prPipeOpen(struct VMGlobals *g, int numArgsPushed)
 int prPipeClose(struct VMGlobals *g, int numArgsPushed)
 {
 	PyrSlot *a;
+	PyrSlot *b;
 	PyrFile *pfile;
 	FILE *file;
+	pid_t pid;
 
-	a = g->sp;
+	a = g->sp - 1;
+	b = g->sp;
 	pfile = (PyrFile*)slotRawObject(a);
 	file = (FILE*)slotRawPtr(&pfile->fileptr);
 	if (file == NULL) return errNone;
+	pid = (pid_t) slotRawInt(b);
+
 	SetPtr(&pfile->fileptr, NULL);
-	int perr = pclose(file);
+	int perr = sc_pclose(file, pid);
 	SetInt(a, perr);
 	if (perr == -1)
 		return errFailed;
 	return errNone;
 }
-#endif
 
 ////////
 
@@ -1504,88 +1506,6 @@ int prSFOpenRead(struct VMGlobals *g, int numArgsPushed)
 	}
 	return errNone;
 }
-/// copied from SC_World.cpp:
-
-int sampleFormatFromString(const char* name);
-int sampleFormatFromString(const char* name)
-{
-	if (!name) return SF_FORMAT_PCM_16;
-
-	size_t len = strlen(name);
-	if (len < 1) return 0;
-
-	if (name[0] == 'u') {
-		if (len < 5) return 0;
-		if (name[4] == '8') return SF_FORMAT_PCM_U8; // uint8
-		return 0;
-	} else if (name[0] == 'i') {
-		if (len < 4) return 0;
-		if (name[3] == '8') return SF_FORMAT_PCM_S8;      // int8
-		else if (name[3] == '1') return SF_FORMAT_PCM_16; // int16
-		else if (name[3] == '2') return SF_FORMAT_PCM_24; // int24
-		else if (name[3] == '3') return SF_FORMAT_PCM_32; // int32
-	} else if (name[0] == 'f') {
-		return SF_FORMAT_FLOAT; // float
-	} else if (name[0] == 'd') {
-		return SF_FORMAT_DOUBLE; // double
-	} else if (name[0] == 'm' || name[0] == 'u') {
-		return SF_FORMAT_ULAW; // mulaw ulaw
-	} else if (name[0] == 'a') {
-		return SF_FORMAT_ALAW; // alaw
-	}
-	return 0;
-}
-
-int headerFormatFromString(const char *name);
-int headerFormatFromString(const char *name)
-{
-	if (!name) return SF_FORMAT_AIFF;
-	if (strcasecmp(name, "AIFF")==0) return SF_FORMAT_AIFF;
-	if (strcasecmp(name, "AIFC")==0) return SF_FORMAT_AIFF;
-	if (strcasecmp(name, "RIFF")==0) return SF_FORMAT_WAV;
-	if (strcasecmp(name, "WAVEX")==0) return SF_FORMAT_WAVEX;
-	if (strcasecmp(name, "WAVE")==0) return SF_FORMAT_WAV;
-	if (strcasecmp(name, "WAV" )==0) return SF_FORMAT_WAV;
-	if (strcasecmp(name, "Sun" )==0) return SF_FORMAT_AU;
-	if (strcasecmp(name, "IRCAM")==0) return SF_FORMAT_IRCAM;
-	if (strcasecmp(name, "NeXT")==0) return SF_FORMAT_AU;
-	if (strcasecmp(name, "raw")==0) return SF_FORMAT_RAW;
-	if (strcasecmp(name, "MAT4")==0) return SF_FORMAT_MAT4;
-	if (strcasecmp(name, "MAT5")==0) return SF_FORMAT_MAT5;
-	if (strcasecmp(name, "PAF")==0) return SF_FORMAT_PAF;
-	if (strcasecmp(name, "SVX")==0) return SF_FORMAT_SVX;
-	if (strcasecmp(name, "NIST")==0) return SF_FORMAT_NIST;
-	if (strcasecmp(name, "VOC")==0) return SF_FORMAT_VOC;
-	if (strcasecmp(name, "W64")==0) return SF_FORMAT_W64;
-	if (strcasecmp(name, "PVF")==0) return SF_FORMAT_PVF;
-	if (strcasecmp(name, "XI")==0) return SF_FORMAT_XI;
-	if (strcasecmp(name, "HTK")==0) return SF_FORMAT_HTK;
-	if (strcasecmp(name, "SDS")==0) return SF_FORMAT_SDS;
-	if (strcasecmp(name, "AVR")==0) return SF_FORMAT_AVR;
-	if (strcasecmp(name, "SD2")==0) return SF_FORMAT_SD2;
-	if (strcasecmp(name, "FLAC")==0) return SF_FORMAT_FLAC;
-	if (strcasecmp(name, "CAF")==0) return SF_FORMAT_CAF;
-// TODO allow other platforms to know vorbis once libsndfile 1.0.18 is established
-#if SC_DARWIN || SC_WIN32
-	if (strcasecmp(name, "VORBIS")==0) return SF_FORMAT_VORBIS;
-#endif
-	if (strcasecmp(name, "RF64")==0) return SF_FORMAT_RF64;
-	return 0;
-}
-
-
-int sndfileFormatInfoFromStrings(struct SF_INFO *info, const char *headerFormatString, const char *sampleFormatString)
-{
-	int headerFormat = headerFormatFromString(headerFormatString);
-	if (!headerFormat) return errWrongType;
-
-	int sampleFormat = sampleFormatFromString(sampleFormatString);
-	if (!sampleFormat) return errWrongType;
-
-	info->format = (unsigned int)(headerFormat | sampleFormat);
-	return errNone;
-}
-// end copy
 
 int prSFOpenWrite(struct VMGlobals *g, int numArgsPushed);
 int prSFOpenWrite(struct VMGlobals *g, int numArgsPushed)
@@ -1646,6 +1566,8 @@ int prSFOpenWrite(struct VMGlobals *g, int numArgsPushed)
 	slotIntVal(slotRawObject(a)->slots + 5, &info.samplerate);
 
 	file = sf_open(filename, SFM_WRITE, &info);
+	sf_command(file, SFC_SET_CLIPPING, NULL, SF_TRUE);
+
 	if (file) {
 		SetPtr(slotRawObject(a)->slots+0, file);
 		SetTrue(a);
@@ -2025,10 +1947,8 @@ void initFilePrimitives()
 	definePrimitive(base, index++, "_SFSeek", prSFSeek, 3, 0);
 	definePrimitive(base, index++, "_SFHeaderInfoString", prSFHeaderInfoString, 1, 0);
 
-#ifndef SC_WIN32
 	definePrimitive(base, index++, "_PipeOpen", prPipeOpen, 3, 0);
-	definePrimitive(base, index++, "_PipeClose", prPipeClose, 1, 0);
-#endif
+	definePrimitive(base, index++, "_PipeClose", prPipeClose, 2, 0);
 
 	definePrimitive(base, index++, "_FileDelete", prFileDelete, 2, 0);
 	definePrimitive(base, index++, "_FileMTime", prFileMTime, 2, 0);
