@@ -57,7 +57,7 @@
 #include "PyrSlot.h"
 #include "VMGlobals.h"
 #include "SC_DirUtils.h"   // for gIdeName
-#include "SC_LibraryConfig.h"
+#include "SC_LanguageConfig.hpp"
 
 #define STDIN_FD 0
 
@@ -247,14 +247,10 @@ int SC_TerminalClient::run(int argc, char** argv)
 	opt.mArgv = argv;
 
 	// read library configuration file
-	if (opt.mLibraryConfigFile) {
-		int argLength = strlen(opt.mLibraryConfigFile);
-		if (strcmp(opt.mLibraryConfigFile + argLength - 5, ".yaml"))
-			SC_LanguageConfig::readLibraryConfig(opt.mLibraryConfigFile);
-		else
-			SC_LanguageConfig::readLibraryConfigYAML(opt.mLibraryConfigFile);
-	} else
-		SC_LanguageConfig::readDefaultLibraryConfig();
+	if (opt.mLibraryConfigFile)
+		SC_LanguageConfig::setConfigFile(opt.mLibraryConfigFile);
+
+	SC_LanguageConfig::readLibraryConfig();
 
 	// initialize runtime
 	initRuntime(opt);
@@ -393,6 +389,7 @@ void SC_TerminalClient::commandLoop()
 
 		while ( mSignals ) {
 			int sig = mSignals;
+			mSignals = 0;
 
 			unlockSignal();
 
@@ -400,10 +397,6 @@ void SC_TerminalClient::commandLoop()
 				//postfl("input\n");
 				lockInput();
 				interpretInput();
-				// clear input signal, as we've processed anything signalled so far.
-				lockSignal();
-				mSignals &= ~sig_input;
-				unlockSignal();
 				unlockInput();
 			}
 
@@ -412,11 +405,6 @@ void SC_TerminalClient::commandLoop()
 				double secs;
 				lock();
 				haveNext = tickLocked( &secs );
-				// clear scheduler signal, as we've processed all items scheduled up to this time.
-				// and will enter the wait according to schedule.
-				lockSignal();
-				mSignals &= ~sig_sched;
-				unlockSignal();
 				unlock();
 
 				flush();
@@ -427,16 +415,10 @@ void SC_TerminalClient::commandLoop()
 
 			if (sig & sig_stop) {
 				stopMain();
-				lockSignal();
-				mSignals &= ~sig_stop;
-				unlockSignal();
 			}
 
 			if (sig & sig_recompile) {
 				recompileLibrary();
-				lockSignal();
-				mSignals &= ~sig_recompile;
-				unlockSignal();
 			}
 
 			lockSignal();
