@@ -29,11 +29,7 @@
 
 #include "SC_PlugIn.h"
 
-#if defined (__GNUC__) && !defined(__clang__)
-#define inline_functions __attribute__ ((flatten))
-#else
-#define inline_functions
-#endif
+#include "function_attributes.h"
 
 static InterfaceTable *ft;
 
@@ -241,14 +237,31 @@ inline double sc_gloop(double in, double hi)
 		windowGuardFrame = windowFrames - 1;			\
 	} while (0);
 
-#define GET_GRAIN_WIN(WINTYPE)							\
-	do {												\
-		if (WINTYPE >= unit->mWorld->mNumSndBufs) {		\
-			Print("Envelope buffer out of range!\n");	\
-			return;										\
-		}												\
-		GET_GRAIN_WIN_RELAXED(WINTYPE)					\
-	} while (0);
+
+static inline bool getGrainWin(Unit * unit, float wintype, SndBuf *& window, const float * & windowData,
+							   uint32 & windowSamples, uint32 & windowFrames, int & windowGuardFrame)
+{
+	if (wintype >= unit->mWorld->mNumSndBufs) {
+		Print("Envelope buffer out of range!\n");
+		return false;
+	}
+
+	assert(wintype < unit->mWorld->mNumSndBufs);
+
+	if (wintype < 0)
+		return true; // use default hann window
+
+	window = unit->mWorld->mSndBufs + (int)wintype;
+	windowData = window->data;
+	if (!windowData)
+		return false;
+
+	windowSamples = window->samples;
+	windowFrames = window->frames;
+	windowGuardFrame = windowFrames - 1;
+
+	return true;
+}
 
 #define GRAIN_LOOP_BODY_4										\
 		float amp = y1 * y1;									\
@@ -534,8 +547,8 @@ inline void GrainIn_next_start_new(GrainIn * unit, int inNumSamples, int positio
 
 	float winType = grain_in_at<full_rate>(unit, 4, position);
 	DECLARE_WINDOW
-	GET_GRAIN_WIN(winType)
-	if (winType >= 0 && (windowData == NULL))
+	bool success = getGrainWin(unit, winType, window, windowData, windowSamples, windowFrames, windowGuardFrame);
+	if (!success)
 		return;
 
 	GrainInG *grain = unit->mGrains + unit->mNumActive++;
@@ -683,8 +696,8 @@ inline void GrainSin_next_start_new(GrainSin * unit, int inNumSamples, int posit
 
 	float winType = grain_in_at<full_rate>(unit, 4, position);
 	DECLARE_WINDOW
-	GET_GRAIN_WIN(winType)
-	if (winType >= 0 && (windowData == NULL))
+	bool success = getGrainWin(unit, winType, window, windowData, windowSamples, windowFrames, windowGuardFrame);
+	if (!success)
 		return;
 
 	GrainSinG *grain = unit->mGrains + unit->mNumActive++;
@@ -850,8 +863,8 @@ inline void GrainFM_next_start_new(GrainFM * unit, int inNumSamples, int positio
 
 	float winType = grain_in_at<full_rate>(unit, 6, position);
 	DECLARE_WINDOW
-	GET_GRAIN_WIN(winType)
-	if (winType >= 0 && (windowData == NULL))
+	bool success = getGrainWin(unit, winType, window, windowData, windowSamples, windowFrames, windowGuardFrame);
+	if (!success)
 		return;
 
 	GrainFMG *grain = unit->mGrains + unit->mNumActive++;
@@ -1068,12 +1081,10 @@ void GrainFM_Dtor(GrainFM *unit)
 
 static inline bool GrainBuf_grain_cleanup(GrainBuf * unit, GrainBufG * grain)
 {
-	if (grain->counter <= 0)
-	{
+	if (grain->counter <= 0) {
 		*grain = unit->mGrains[--unit->mNumActive]; // remove grain
 		return true;
-	}
-	else
+	} else
 		return false;
 }
 
@@ -1090,10 +1101,10 @@ static inline void GrainBuf_next_play_active(GrainBuf *unit, int inNumSamples)
 
 		GRAIN_BUF
 
-		if (!bufData) {
+		if (!bufData || (bufChannels != 1)) {
 			grain->counter -= inNumSamples;
 			if (!GrainBuf_grain_cleanup(unit, grain))
-                ++i;
+				++i;
 			continue;
 		}
 
@@ -1104,6 +1115,8 @@ static inline void GrainBuf_next_play_active(GrainBuf *unit, int inNumSamples)
 		DECLARE_WINDOW
 		GET_GRAIN_AMP_PARAMS
 
+
+
 		// begin add //
 		float pan2 = 0.f;
 		float *out2;
@@ -1136,13 +1149,13 @@ static inline void GrainBuf_next_start_new(GrainBuf *unit, int inNumSamples, int
 		return;
 	}
 
-	GrainBufG *grain = unit->mGrains + unit->mNumActive++;
 	float winType = grain_in_at<full_rate>(unit, 7, position);
 	DECLARE_WINDOW
-	GET_GRAIN_WIN(winType)
-	if (winType >= 0 && (windowData == NULL))
+	bool success = getGrainWin(unit, winType, window, windowData, windowSamples, windowFrames, windowGuardFrame);
+	if (!success)
 		return;
 
+	GrainBufG *grain = unit->mGrains + unit->mNumActive++;
 	int32 bufnum = grain_in_at<full_rate>(unit, 2, position);
 	grain->bufnum = bufnum;
 
@@ -1157,12 +1170,17 @@ static inline void GrainBuf_next_start_new(GrainBuf *unit, int inNumSamples, int
 	float bufRateScale = bufSampleRate * SAMPLEDUR;
 	double loopMax = (double)bufFrames;
 
+	double rate = grain->rate = grain_in_at<full_rate>(unit, 3, position) * bufRateScale;
+	double phase = grain_in_at<full_rate>(unit, 4, position) * bufFrames;
+	if (sc_isnan(phase)) {
+		GrainBuf_grain_cleanup(unit, grain);
+		return;
+	}
+
 	double counter = grain_in_at<full_rate>(unit, 1, position) * SAMPLERATE;
 	counter = sc_max(4., counter);
 	grain->counter = (int)counter;
 
-	double rate = grain->rate = grain_in_at<full_rate>(unit, 3, position) * bufRateScale;
-	double phase = grain_in_at<full_rate>(unit, 4, position) * bufFrames;
 	grain->interp = (int)grain_in_at<full_rate>(unit, 5, position);
 	grain->winType = winType;
 
@@ -1222,22 +1240,22 @@ static void GrainBuf_next_k(GrainBuf * unit, int inNumSamples)
 	unit->curtrig = trig;
 }
 
-inline_functions static void GrainBuf_next_k_1(GrainBuf * unit, int inNumSamples)
+FLATTEN static void GrainBuf_next_k_1(GrainBuf * unit, int inNumSamples)
 {
 	GrainBuf_next_k<true>(unit, inNumSamples);
 }
 
-inline_functions static void GrainBuf_next_k_2(GrainBuf * unit, int inNumSamples)
+FLATTEN static void GrainBuf_next_k_2(GrainBuf * unit, int inNumSamples)
 {
 	GrainBuf_next_k<false>(unit, inNumSamples);
 }
 
-inline_functions static void GrainBuf_next_a_1(GrainBuf * unit, int inNumSamples)
+FLATTEN static void GrainBuf_next_a_1(GrainBuf * unit, int inNumSamples)
 {
 	GrainBuf_next_a<true>(unit, inNumSamples);
 }
 
-inline_functions static void GrainBuf_next_a_2(GrainBuf * unit, int inNumSamples)
+FLATTEN static void GrainBuf_next_a_2(GrainBuf * unit, int inNumSamples)
 {
 	GrainBuf_next_a<false>(unit, inNumSamples);
 }
