@@ -27,6 +27,7 @@
 #include "SC_UnitSpec.h"
 #include "SC_UnitDef.h"
 #include "SC_HiddenWorld.h"
+#include "SC_WorldOptions.h"
 #ifndef _MSC_VER
 #include <dirent.h>
 #endif //_MSC_VER
@@ -88,6 +89,14 @@ void ReadNodeDefName(char*& buffer, int32* name)
 
 void ParamSpec_Read(ParamSpec* inParamSpec, char*& buffer);
 void ParamSpec_Read(ParamSpec* inParamSpec, char*& buffer)
+{
+	ReadName(buffer, inParamSpec->mName);
+	inParamSpec->mIndex = readInt32_be(buffer);
+	inParamSpec->mHash = Hash(inParamSpec->mName);
+}
+
+void ParamSpec_ReadVer1(ParamSpec* inParamSpec, char*& buffer);
+void ParamSpec_ReadVer1(ParamSpec* inParamSpec, char*& buffer)
 {
 	ReadName(buffer, inParamSpec->mName);
 	inParamSpec->mIndex = readInt16_be(buffer);
@@ -96,6 +105,15 @@ void ParamSpec_Read(ParamSpec* inParamSpec, char*& buffer)
 
 void InputSpec_Read(InputSpec* inInputSpec, char*& buffer);
 void InputSpec_Read(InputSpec* inInputSpec, char*& buffer)
+{
+	inInputSpec->mFromUnitIndex = readInt32_be(buffer);
+	inInputSpec->mFromOutputIndex = readInt32_be(buffer);
+
+	inInputSpec->mWireIndex = -1;
+}
+
+void InputSpec_ReadVer1(InputSpec* inInputSpec, char*& buffer);
+void InputSpec_ReadVer1(InputSpec* inInputSpec, char*& buffer)
 {
 	inInputSpec->mFromUnitIndex = (int16)readInt16_be(buffer);
 	inInputSpec->mFromOutputIndex = (int16)readInt16_be(buffer);
@@ -107,7 +125,6 @@ void OutputSpec_Read(OutputSpec* inOutputSpec, char*& buffer);
 void OutputSpec_Read(OutputSpec* inOutputSpec, char*& buffer)
 {
 	inOutputSpec->mCalcRate = readInt8(buffer);
-
 	inOutputSpec->mWireIndex = -1;
 	inOutputSpec->mBufferIndex = -1;
 	inOutputSpec->mNumConsumers = 0;
@@ -128,8 +145,8 @@ void UnitSpec_Read(UnitSpec* inUnitSpec, char*& buffer)
 	}
 	inUnitSpec->mCalcRate = readInt8(buffer);
 
-	inUnitSpec->mNumInputs = readInt16_be(buffer);
-	inUnitSpec->mNumOutputs = readInt16_be(buffer);
+	inUnitSpec->mNumInputs = readInt32_be(buffer);
+	inUnitSpec->mNumOutputs = readInt32_be(buffer);
 	inUnitSpec->mSpecialIndex = readInt16_be(buffer);
 	inUnitSpec->mInputSpec = (InputSpec*)malloc(sizeof(InputSpec) * inUnitSpec->mNumInputs);
 	inUnitSpec->mOutputSpec = (OutputSpec*)malloc(sizeof(OutputSpec) * inUnitSpec->mNumOutputs);
@@ -139,11 +156,42 @@ void UnitSpec_Read(UnitSpec* inUnitSpec, char*& buffer)
 	for (uint32 i=0; i<inUnitSpec->mNumOutputs; ++i) {
 		OutputSpec_Read(inUnitSpec->mOutputSpec + i, buffer);
 	}
-	int numPorts = inUnitSpec->mNumInputs + inUnitSpec->mNumOutputs;
+	uint64 numPorts = inUnitSpec->mNumInputs + inUnitSpec->mNumOutputs;
+	inUnitSpec->mAllocSize = inUnitSpec->mUnitDef->mAllocSize + numPorts * (sizeof(Wire*) +  sizeof(float*));
+}
+
+void UnitSpec_ReadVer1(UnitSpec* inUnitSpec, char*& buffer);
+void UnitSpec_ReadVer1(UnitSpec* inUnitSpec, char*& buffer)
+{
+	int32 name[kSCNameLen];
+	ReadName(buffer, name);
+
+	inUnitSpec->mUnitDef = GetUnitDef(name);
+	if (!inUnitSpec->mUnitDef) {
+		char str[256];
+		sprintf(str, "UGen '%s' not installed.", (char*)name);
+		throw std::runtime_error(str);
+		return;
+	}
+	inUnitSpec->mCalcRate = readInt8(buffer);
+
+	inUnitSpec->mNumInputs = readInt16_be(buffer);
+	inUnitSpec->mNumOutputs = readInt16_be(buffer);
+	inUnitSpec->mSpecialIndex = readInt16_be(buffer);
+	inUnitSpec->mInputSpec = (InputSpec*)malloc(sizeof(InputSpec) * inUnitSpec->mNumInputs);
+	inUnitSpec->mOutputSpec = (OutputSpec*)malloc(sizeof(OutputSpec) * inUnitSpec->mNumOutputs);
+	for (uint32 i=0; i<inUnitSpec->mNumInputs; ++i) {
+		InputSpec_ReadVer1(inUnitSpec->mInputSpec + i, buffer);
+	}
+	for (uint32 i=0; i<inUnitSpec->mNumOutputs; ++i) {
+		OutputSpec_Read(inUnitSpec->mOutputSpec + i, buffer);
+	}
+	uint64 numPorts = inUnitSpec->mNumInputs + inUnitSpec->mNumOutputs;
 	inUnitSpec->mAllocSize = inUnitSpec->mUnitDef->mAllocSize + numPorts * (sizeof(Wire*) +  sizeof(float*));
 }
 
 GraphDef* GraphDef_Read(World *inWorld, char*& buffer, GraphDef* inList, int32 inVersion);
+GraphDef* GraphDef_ReadVer1(World *inWorld, char*& buffer, GraphDef* inList, int32 inVersion);
 
 GraphDef* GraphDefLib_Read(World *inWorld, char* buffer, GraphDef* inList);
 GraphDef* GraphDefLib_Read(World *inWorld, char* buffer, GraphDef* inList)
@@ -152,14 +200,32 @@ GraphDef* GraphDefLib_Read(World *inWorld, char* buffer, GraphDef* inList)
 	if (magic != (('S'<<24)|('C'<<16)|('g'<<8)|'f') /*'SCgf'*/) return inList;
 
 	int32 version = readInt32_be(buffer);
-	if (version > 1) return inList;
 
-	uint32 numDefs = readInt16_be(buffer);
+	uint32 numDefs, i;
+	switch (version) {
+		case 2:
+			numDefs = readInt16_be(buffer);
 
-	for (uint32 i=0; i<numDefs; ++i) {
-		inList = GraphDef_Read(inWorld, buffer, inList, version);
+			for (i=0; i<numDefs; ++i) {
+				inList = GraphDef_Read(inWorld, buffer, inList, version);
+			}
+			return inList;
+			break;
+		case 1:
+		case 0:
+			numDefs = readInt16_be(buffer);
+
+			for (i=0; i<numDefs; ++i) {
+				inList = GraphDef_ReadVer1(inWorld, buffer, inList, version); // handles 1 and 0
+			}
+			return inList;
+			break;
+
+		default:
+			return inList;
+			break;
 	}
-	return inList;
+
 }
 
 
@@ -182,6 +248,7 @@ void GraphDef_ReadVariant(World *inWorld, char*& buffer, GraphDef* inGraphDef, G
 	}
 }
 
+// ver 2
 GraphDef* GraphDef_Read(World *inWorld, char*& buffer, GraphDef* inList, int32 inVersion)
 {
 	int32 name[kSCNodeDefNameLen];
@@ -197,6 +264,115 @@ GraphDef* GraphDef_Read(World *inWorld, char*& buffer, GraphDef* inList, int32 i
 
 	graphDef->mNodeDef.mHash = Hash(graphDef->mNodeDef.mName);
 
+	graphDef->mNumConstants = readInt32_be(buffer);
+
+	graphDef->mConstants = (float*)malloc(graphDef->mNumConstants * sizeof(float));
+	for (uint32 i=0; i<graphDef->mNumConstants; ++i) {
+		graphDef->mConstants[i] = readFloat_be(buffer);
+	}
+
+	graphDef->mNumControls = readInt32_be(buffer);
+	graphDef->mInitialControlValues = (float32*)malloc(sizeof(float32) * graphDef->mNumControls);
+	for (uint32 i=0; i<graphDef->mNumControls; ++i) {
+		graphDef->mInitialControlValues[i] = readFloat_be(buffer);
+	}
+
+	graphDef->mNumParamSpecs = readInt32_be(buffer);
+	if (graphDef->mNumParamSpecs) {
+		int hashTableSize = NEXTPOWEROFTWO(graphDef->mNumParamSpecs);
+		graphDef->mParamSpecTable = new ParamSpecTable(&gMalloc, hashTableSize, false);
+		graphDef->mParamSpecs = (ParamSpec*)malloc(graphDef->mNumParamSpecs * sizeof(ParamSpec));
+		for (uint32 i=0; i<graphDef->mNumParamSpecs; ++i) {
+			ParamSpec *paramSpec = graphDef->mParamSpecs + i;
+			ParamSpec_Read(paramSpec, buffer);
+			graphDef->mParamSpecTable->Add(paramSpec);
+		}
+	} else {
+		// empty table to eliminate test in Graph_SetControl
+		graphDef->mParamSpecTable = new ParamSpecTable(&gMalloc, 4, false);
+		graphDef->mParamSpecs = 0;
+	}
+
+	graphDef->mNumWires = graphDef->mNumConstants;
+	graphDef->mNumUnitSpecs = readInt32_be(buffer);
+	graphDef->mUnitSpecs = (UnitSpec*)malloc(sizeof(UnitSpec) * graphDef->mNumUnitSpecs);
+	graphDef->mNumCalcUnits = 0;
+	for (uint32 i=0; i<graphDef->mNumUnitSpecs; ++i) {
+		UnitSpec *unitSpec = graphDef->mUnitSpecs + i;
+		UnitSpec_Read(unitSpec, buffer);
+
+		switch (unitSpec->mCalcRate)
+		{
+			case calc_ScalarRate :
+				unitSpec->mRateInfo = &inWorld->mBufRate;
+				break;
+			case calc_BufRate :
+				graphDef->mNumCalcUnits++;
+				unitSpec->mRateInfo = &inWorld->mBufRate;
+				break;
+			case calc_FullRate :
+				graphDef->mNumCalcUnits++;
+				unitSpec->mRateInfo = &inWorld->mFullRate;
+				break;
+			case calc_DemandRate :
+				unitSpec->mRateInfo = &inWorld->mBufRate;
+				break;
+		}
+
+		graphDef->mNodeDef.mAllocSize += unitSpec->mAllocSize;
+		graphDef->mNumWires += unitSpec->mNumOutputs;
+	}
+
+	DoBufferColoring(inWorld, graphDef);
+
+	graphDef->mWiresAllocSize = graphDef->mNumWires * sizeof(Wire);
+	graphDef->mUnitsAllocSize = graphDef->mNumUnitSpecs * sizeof(Unit*);
+	graphDef->mCalcUnitsAllocSize = graphDef->mNumCalcUnits * sizeof(Unit*);
+
+	graphDef->mNodeDef.mAllocSize += graphDef->mWiresAllocSize;
+	graphDef->mNodeDef.mAllocSize += graphDef->mUnitsAllocSize;
+	graphDef->mNodeDef.mAllocSize += graphDef->mCalcUnitsAllocSize;
+
+	graphDef->mControlAllocSize = graphDef->mNumControls * sizeof(float);
+	graphDef->mNodeDef.mAllocSize += graphDef->mControlAllocSize;
+
+	graphDef->mMapControlsAllocSize = graphDef->mNumControls * sizeof(float*);
+	graphDef->mNodeDef.mAllocSize += graphDef->mMapControlsAllocSize;
+
+	graphDef->mMapControlRatesAllocSize = graphDef->mNumControls * sizeof(int*);
+	graphDef->mNodeDef.mAllocSize += graphDef->mMapControlRatesAllocSize;
+
+
+	graphDef->mNext = inList;
+	graphDef->mRefCount = 1;
+
+	graphDef->mNumVariants = readInt16_be(buffer);
+	if (graphDef->mNumVariants) {
+		graphDef->mVariants = (GraphDef*)calloc(graphDef->mNumVariants, sizeof(GraphDef));
+		for (uint32 i=0; i<graphDef->mNumVariants; ++i) {
+			GraphDef_ReadVariant(inWorld, buffer, graphDef, graphDef->mVariants + i);
+		}
+	}
+
+	return graphDef;
+}
+
+// ver 0 or 1
+GraphDef* GraphDef_ReadVer1(World *inWorld, char*& buffer, GraphDef* inList, int32 inVersion)
+{
+	int32 name[kSCNodeDefNameLen];
+	ReadNodeDefName(buffer, name);
+
+	GraphDef* graphDef = (GraphDef*)calloc(1, sizeof(GraphDef));
+
+	graphDef->mOriginal = graphDef;
+
+	graphDef->mNodeDef.mAllocSize = sizeof(Graph);
+
+	memcpy((char*)graphDef->mNodeDef.mName, (char*)name, kSCNodeDefNameByteLen);
+
+	graphDef->mNodeDef.mHash = Hash(graphDef->mNodeDef.mName);
+
 	graphDef->mNumConstants = readInt16_be(buffer);
 	graphDef->mConstants = (float*)malloc(graphDef->mNumConstants * sizeof(float));
 	for (uint32 i=0; i<graphDef->mNumConstants; ++i) {
@@ -216,7 +392,7 @@ GraphDef* GraphDef_Read(World *inWorld, char*& buffer, GraphDef* inList, int32 i
 		graphDef->mParamSpecs = (ParamSpec*)malloc(graphDef->mNumParamSpecs * sizeof(ParamSpec));
 		for (uint32 i=0; i<graphDef->mNumParamSpecs; ++i) {
 			ParamSpec *paramSpec = graphDef->mParamSpecs + i;
-			ParamSpec_Read(paramSpec, buffer);
+			ParamSpec_ReadVer1(paramSpec, buffer);
 			graphDef->mParamSpecTable->Add(paramSpec);
 		}
 	} else {
@@ -231,7 +407,7 @@ GraphDef* GraphDef_Read(World *inWorld, char*& buffer, GraphDef* inList, int32 i
 	graphDef->mNumCalcUnits = 0;
 	for (uint32 i=0; i<graphDef->mNumUnitSpecs; ++i) {
 		UnitSpec *unitSpec = graphDef->mUnitSpecs + i;
-		UnitSpec_Read(unitSpec, buffer);
+		UnitSpec_ReadVer1(unitSpec, buffer);
 
 		switch (unitSpec->mCalcRate)
 		{
@@ -327,6 +503,19 @@ void GraphDef_DeleteMsg(World *inWorld, GraphDef *inDef)
 	DeleteGraphDefMsg msg;
 	msg.mDef = inDef;
 	inWorld->hw->mDeleteGraphDefs.Write(msg);
+
+	small_scpacket packet;
+	packet.adds("/d_removed");
+	packet.maketags(1);
+	packet.addtag(',');
+	packet.addtag('s');
+	packet.adds((char*)inDef->mNodeDef.mName);
+
+	ReplyAddress *users = inWorld->hw->mUsers;
+	int numUsers = inWorld->hw->mNumUsers;
+	for (int i=0; i<numUsers; ++i) {
+		SendReply(users+i, packet.data(), packet.size());
+	}
 }
 
 GraphDef* GraphDef_Recv(World *inWorld, char *buffer, GraphDef *inList)
@@ -532,7 +721,7 @@ struct BufColorAllocator
 	BufColorAllocator();
 	~BufColorAllocator();
 
-	int alloc(int count);
+	uint32 alloc(uint32 count);
 	bool release(int inIndex);
 	int NumBufs() { return nextIndex; }
 };
@@ -553,9 +742,9 @@ inline BufColorAllocator::~BufColorAllocator()
 	free(stack);
 }
 
-inline int BufColorAllocator::alloc(int count)
+inline uint32 BufColorAllocator::alloc(uint32 count)
 {
-	int outIndex;
+	uint32 outIndex;
 	if (stackPtr) {
 		outIndex = stack[--stackPtr];
 	} else {
@@ -586,8 +775,7 @@ inline bool BufColorAllocator::release(int inIndex)
 
 static void ReleaseInputBuffers(GraphDef *inGraphDef, UnitSpec *unitSpec, BufColorAllocator& bufColor)
 {
-	//scprintf("ReleaseInputBuffers %s numinputs %d\n", unitSpec->mUnitDef->mUnitDefName, unitSpec->mNumInputs);
-	for (int i=unitSpec->mNumInputs-1; i>=0; --i) {
+	for (int64 i=(int64)(unitSpec->mNumInputs)-1; i>=0; --i) {
 		InputSpec *inputSpec = unitSpec->mInputSpec + i;
 		if (inputSpec->mFromUnitIndex >= 0) {
 			UnitSpec *outUnit = inGraphDef->mUnitSpecs + inputSpec->mFromUnitIndex;
@@ -613,14 +801,14 @@ static void ReleaseInputBuffers(GraphDef *inGraphDef, UnitSpec *unitSpec, BufCol
 	}
 }
 
-void AllocOutputBuffers(UnitSpec *unitSpec, BufColorAllocator& bufColor, int& wireIndexCtr)
+static void AllocOutputBuffers(UnitSpec *unitSpec, BufColorAllocator& bufColor, int32& wireIndexCtr)
 {
 	//scprintf("AllocOutputBuffers %s numoutputs %d\n", unitSpec->mUnitDef->mUnitDefName, unitSpec->mNumOutputs);
 	for (uint32 i=0; i<unitSpec->mNumOutputs; ++i) {
 		OutputSpec *outputSpec = unitSpec->mOutputSpec + i;
 		outputSpec->mWireIndex = wireIndexCtr++;
 		if (outputSpec->mCalcRate == calc_FullRate) {
-			int bufIndex = bufColor.alloc(outputSpec->mNumConsumers);
+			uint32 bufIndex = bufColor.alloc(outputSpec->mNumConsumers);
 			outputSpec->mBufferIndex = bufIndex;
 		}
 	}
@@ -644,7 +832,7 @@ void DoBufferColoring(World *inWorld, GraphDef *inGraphDef)
 	// buffer coloring
 	{
 		BufColorAllocator bufColor;
-		int wireIndexCtr = inGraphDef->mNumConstants;
+		int32 wireIndexCtr = inGraphDef->mNumConstants; // mNumConstants is a uint32, but limited to int32 in OSC
 		for (uint32 j=0; j<inGraphDef->mNumUnitSpecs; ++j) {
 			UnitSpec *unitSpec = inGraphDef->mUnitSpecs + j;
 			if (unitSpec->mUnitDef->mFlags & kUnitDef_CantAliasInputsToOutputs) {
