@@ -28,6 +28,8 @@
 #include "SC_Sem.h"
 #include "SC_DirUtils.h"
 #include "SC_StringParser.h"
+#include "../../common/SC_SndFileHelpers.hpp"
+#include "SC_WorldOptions.h"
 
 #define GET_COMPLETION_MSG(msg) \
 	mMsgSize = msg.getbsize(); \
@@ -319,7 +321,7 @@ bool BufAllocCmd::Stage3()
 
 void BufAllocCmd::Stage4()
 {
-	free(mFreeData);
+	zfree(mFreeData);
 	SendDoneWithIntValue("/b_alloc", mBufIndex);
 }
 
@@ -385,7 +387,7 @@ bool BufGenCmd::Stage3()
 
 void BufGenCmd::Stage4()
 {
-	free(mFreeData);
+	zfree(mFreeData);
 	SendDoneWithIntValue("/b_gen", mBufIndex);
 }
 
@@ -440,7 +442,7 @@ bool BufFreeCmd::Stage3()
 
 void BufFreeCmd::Stage4()
 {
-	free(mFreeData);
+	zfree(mFreeData);
 	SendDoneWithIntValue("/b_free", mBufIndex);
 }
 
@@ -537,8 +539,8 @@ bool BufAllocReadCmd::Stage2()
 	memset(&fileinfo, 0, sizeof(fileinfo));
 	SNDFILE* sf = sf_open(mFilename, SFM_READ, &fileinfo);
 	if (!sf) {
-		char str[256];
-		sprintf(str, "File '%s' could not be opened.\n", mFilename);
+		char str[512];
+		sprintf(str, "File '%s' could not be opened: %s\n", mFilename, sf_strerror(NULL));
 		SendFailureWithBufnum(&mReplyAddress, "/b_allocRead", str, mBufIndex);	//SendFailure(&mReplyAddress, "/b_allocRead", str);
 		scprintf(str);
 		return false;
@@ -575,7 +577,7 @@ bool BufAllocReadCmd::Stage3()
 
 void BufAllocReadCmd::Stage4()
 {
-	free(mFreeData);
+	zfree(mFreeData);
 	SendDoneWithIntValue("/b_allocRead", mBufIndex);
 }
 
@@ -637,16 +639,16 @@ bool BufReadCmd::Stage2()
 
 	SNDFILE* sf = sf_open(mFilename, SFM_READ, &fileinfo);
 	if (!sf) {
-		char str[256];
-		sprintf(str, "File '%s' could not be opened.\n", mFilename);
+		char str[512];
+		sprintf(str, "File '%s' could not be opened: %s\n", mFilename, sf_strerror(NULL));
 		SendFailureWithBufnum(&mReplyAddress, "/b_read", str, mBufIndex); //SendFailure(&mReplyAddress, "/b_read", str);
 		scprintf(str);
 		return false;
 	}
 	if (fileinfo.channels != buf->channels) {
-		char str[256];
+		char str[512];
 		sf_close(sf);
-		sprintf(str, "channel mismatch. File'%s' has %d channels. Buffer has %d channels.\n", mFilename, fileinfo.channels, buf->channels);
+		sprintf(str, "Channel mismatch. File '%s' has %d channels. Buffer has %d channels.\n", mFilename, fileinfo.channels, buf->channels);
 		SendFailureWithBufnum(&mReplyAddress, "/b_read", str, mBufIndex); //SendFailure(&mReplyAddress, "/b_read", str);
 		scprintf(str);
 		return false;
@@ -800,8 +802,8 @@ bool BufAllocReadChannelCmd::Stage2()
 	memset(&fileinfo, 0, sizeof(fileinfo));
 	SNDFILE* sf = sf_open(mFilename, SFM_READ, &fileinfo);
 	if (!sf) {
-		char str[256];
-		sprintf(str, "File '%s' could not be opened.\n", mFilename);
+		char str[512];
+		sprintf(str, "File '%s' could not be opened: %s\n", mFilename, sf_strerror(NULL));
 		SendFailureWithBufnum(&mReplyAddress, "/b_allocReadChannel", str, mBufIndex); //SendFailure(&mReplyAddress, "/b_allocRead", str);
 		scprintf(str);
 		return false;
@@ -862,7 +864,7 @@ bool BufAllocReadChannelCmd::Stage3()
 
 void BufAllocReadChannelCmd::Stage4()
 {
-	free(mFreeData);
+	zfree(mFreeData);
 	SendDoneWithIntValue("/b_allocReadChannel", mBufIndex);
 }
 
@@ -926,8 +928,8 @@ bool BufReadChannelCmd::Stage2()
 
 	SNDFILE* sf = sf_open(mFilename, SFM_READ, &fileinfo);
 	if (!sf) {
-		char str[256];
-		sprintf(str, "File '%s' could not be opened.\n", mFilename);
+		char str[512];
+		sprintf(str, "File '%s' could not be opened: %s\n", mFilename, sf_strerror(NULL));
 		SendFailureWithBufnum(&mReplyAddress, "/b_readChannel", str, mBufIndex); //SendFailure(&mReplyAddress, "/b_read", str);
 		scprintf(str);
 		return false;
@@ -1020,12 +1022,6 @@ BufWriteCmd::BufWriteCmd(World *inWorld, ReplyAddress *inReplyAddress)
 struct SF_INFO {};
 #endif
 
-
-extern "C" {
-int sndfileFormatInfoFromStrings(SF_INFO *info, const char *headerFormatString, const char *sampleFormatString);
-}
-
-
 int BufWriteCmd::Init(char *inData, int inSize)
 {
 #ifdef NO_LIBSNDFILE
@@ -1083,10 +1079,8 @@ bool BufWriteCmd::Stage2()
 
 	SNDFILE* sf = sf_open(mFilename, SFM_WRITE, &mFileInfo);
 	if (!sf) {
-		char sferr[256];
-		char str[256];
-		sf_error_str(NULL, sferr, 256);
-		sprintf(str, "File '%s' could not be opened. '%s'\n", mFilename, sferr);
+		char str[512];
+		sprintf(str, "File '%s' could not be opened: %s\n", mFilename, sf_strerror(NULL));
 		SendFailureWithBufnum(&mReplyAddress, "/b_write", str, mBufIndex); //SendFailure(&mReplyAddress, "/b_write", str);
 		scprintf(str);
 		return false;
@@ -1096,6 +1090,8 @@ bool BufWriteCmd::Stage2()
 
 	if (mNumFrames > framesToEnd) mNumFrames = framesToEnd;
 
+	sf_command(sf, SFC_SET_CLIPPING, NULL, SF_TRUE); // choose clipping rather than wraparound for integer-format files
+
 	if (mNumFrames > 0) {
 		sf_writef_float(sf, buf->data + (mBufOffset * buf->channels), mNumFrames);
 	}
@@ -1199,6 +1195,8 @@ bool AudioQuitCmd::Stage3()
 	scprintf("/quit : quit not allowed in AU host\n");
 	return false;
 #else
+	if (mWorld->hw->mShmem)
+		mWorld->hw->mShmem->disconnect();
 	return true;
 #endif
 }
