@@ -38,6 +38,9 @@
 #include "SC_Unit.h"
 #include "clz.h"
 #include "SC_fftlib.h"
+#include "../../common/SC_SndFileHelpers.hpp"
+
+#include <boost/math/constants/constants.hpp>
 
 // undefine the shadowed scfft functions
 #undef scfft_create
@@ -51,6 +54,8 @@ spin_lock log_guard; // needs to be acquired for logging from the helper threads
 
 namespace {
 
+spin_lock rt_pool_guard;
+
 inline Node * as_Node(server_node * node)
 {
     if (node == NULL)
@@ -318,8 +323,7 @@ void release_scope_buffer(World *inWorld, ScopeBufferHnd &hnd)
 } /* namespace */
 } /* namespace nova */
 
-extern "C"
-{
+extern "C" {
 
 bool define_unit(const char *inUnitClassName, size_t inAllocSize,
                  UnitCtorFunc inCtor, UnitDtorFunc inDtor, uint32 inFlags)
@@ -351,7 +355,6 @@ bool define_unitcmd(const char * unitClassName, const char * cmdName, UnitCmdFun
     return nova::sc_factory->register_ugen_command_function(unitClassName, cmdName, inFunc);
 }
 
-
 bool define_plugincmd(const char * name, PlugInCmdFunc func, void * user_data)
 {
     return nova::sc_factory->register_cmd_plugin(name, func, user_data);
@@ -359,6 +362,7 @@ bool define_plugincmd(const char * name, PlugInCmdFunc func, void * user_data)
 
 void * rt_alloc(World * dummy, size_t size)
 {
+    nova::spin_lock::scoped_lock lock(nova::rt_pool_guard);
     if (size)
         return nova::rt_pool.malloc(size);
     else
@@ -367,11 +371,13 @@ void * rt_alloc(World * dummy, size_t size)
 
 void * rt_realloc(World * dummy, void * ptr, size_t size)
 {
+    nova::spin_lock::scoped_lock lock(nova::rt_pool_guard);
     return nova::rt_pool.realloc(ptr, size);
 }
 
 void rt_free(World * dummy, void * ptr)
 {
+    nova::spin_lock::scoped_lock lock(nova::rt_pool_guard);
     if (ptr)
         nova::rt_pool.free(ptr);
 }
@@ -562,7 +568,7 @@ int do_asynchronous_command(World *inWorld, void* replyAddr, const char* cmdName
                                             stage2, stage3, stage4, cleanup,
                                             completionMsgSize, completionMsgData);
     return 0;
-};
+}
 
 } /* extern "C" */
 
@@ -574,7 +580,7 @@ inline void initialize_rate(Rate & rate, double sample_rate, int blocksize)
 {
     rate.mSampleRate = sample_rate;
     rate.mSampleDur = 1. / sample_rate;
-    rate.mRadiansPerSample = 2 * M_PI / sample_rate;
+    rate.mRadiansPerSample = 2 * boost::math::constants::pi<double>() / sample_rate;
 
     rate.mBufLength = blocksize;
     rate.mBufDuration = blocksize / sample_rate;
@@ -612,6 +618,13 @@ void sc_plugin_interface::initialize(server_arguments const & args, float * cont
     sc_interface.fPrint = &print;
     sc_interface.fDoneAction = &done_action;
 
+    /* sndfile functions */
+#ifdef NO_LIBSNDFILE
+    sc_interface.fSndFileFormatInfoFromStrings = NULL;
+#else
+    sc_interface.fSndFileFormatInfoFromStrings = &sndfileFormatInfoFromStrings;
+#endif
+
     /* wave tables */
     sc_interface.mSine = gSine;
     sc_interface.mCosecant = gInvSine;
@@ -683,7 +696,7 @@ void sc_plugin_interface::initialize(server_arguments const & args, float * cont
     memset(world.mSndBufUpdates, 0, world.mNumSndBufs*sizeof(SndBufUpdates));
     world.mBufCounter = 0;
 
-    async_buffer_guards.reset(new boost::mutex[world.mNumSndBufs]);
+    async_buffer_guards.reset(new std::mutex[world.mNumSndBufs]);
 
     /* audio settings */
     world.mBufLength = args.blocksize;
@@ -696,6 +709,27 @@ void sc_plugin_interface::initialize(server_arguments const & args, float * cont
     world.mNumOutputs = args.output_channels;
 
     world.mRealTime = !args.non_rt;
+
+    /* rngs */
+    world.mNumRGens = args.rng_count;
+    world.mRGen = new RGen[world.mNumRGens];
+    std::vector<std::uint32_t> seeds(world.mNumRGens);
+
+    try {
+        std::random_device rd;
+        std::seed_seq seq({ rd(), rd(), rd() });
+        seq.generate(seeds.begin(), seeds.end());
+    } catch (...) {
+        auto now = std::chrono::high_resolution_clock::now();
+
+        auto seconds = std::chrono::duration_cast<std::chrono::seconds>(now.time_since_epoch());
+
+        std::seed_seq seq({ seconds.count() });
+        seq.generate(seeds.begin(), seeds.end());
+    }
+
+    for (int i=0; i<world.mNumRGens; ++i)
+        world.mRGen[i].init(seeds[i]);
 }
 
 void sc_plugin_interface::reset_sampling_rate(int sr)
@@ -709,19 +743,24 @@ void sc_plugin_interface::reset_sampling_rate(int sr)
 
 void sc_done_action_handler::update_nodegraph(void)
 {
-    std::for_each(done_nodes.begin(), done_nodes.end(), boost::bind(&nova_server::free_node, instance, _1));
+    for (server_node * node : done_nodes)
+        instance->free_node(node);
     done_nodes.clear();
 
-    std::for_each(resume_nodes.begin(), resume_nodes.end(), boost::bind(&nova_server::node_resume, instance, _1));
+    for (server_node * node : resume_nodes)
+        instance->node_resume(node);
     resume_nodes.clear();
 
-    std::for_each(pause_nodes.begin(), pause_nodes.end(), boost::bind(&nova_server::node_pause, instance, _1));
+    for (server_node * node : pause_nodes)
+        instance->node_pause(node);
     pause_nodes.clear();
 
-    std::for_each(freeDeep_nodes.begin(), freeDeep_nodes.end(), boost::bind(&nova_server::group_free_deep, instance, _1));
+    for (abstract_group * group : freeDeep_nodes)
+        instance->group_free_deep(group);
     freeDeep_nodes.clear();
 
-    std::for_each(freeAll_nodes.begin(), freeAll_nodes.end(), boost::bind(&nova_server::group_free_all, instance, _1));
+    for (abstract_group * group : freeAll_nodes)
+        instance->group_free_all(group);
     freeAll_nodes.clear();
 }
 
@@ -733,11 +772,11 @@ sc_plugin_interface::~sc_plugin_interface(void)
     delete[] world.mSndBufs;
     delete[] world.mSndBufsNonRealTimeMirror;
     delete[] world.mSndBufUpdates;
+    delete[] world.mRGen;
     delete world.mNRTLock;
 }
 
-namespace
-{
+namespace {
 
 sample * allocate_buffer(size_t samples)
 {
@@ -861,11 +900,11 @@ SndBuf * sc_plugin_interface::allocate_buffer(uint32_t index, uint32_t frames, u
     return buf;
 }
 
-int sc_plugin_interface::buffer_read_alloc(uint32_t index, const char * filename, uint32_t start, uint32_t frames)
+void sc_plugin_interface::buffer_read_alloc(uint32_t index, const char * filename, uint32_t start, uint32_t frames)
 {
     SndfileHandle f(filename);
-    if (!f)
-        return -1; /* file cannot be opened */
+    if (f.rawHandle() == nullptr)
+        throw std::runtime_error(f.strError());
 
     const size_t sf_frames = f.frames();
 
@@ -880,22 +919,21 @@ int sc_plugin_interface::buffer_read_alloc(uint32_t index, const char * filename
 
     f.seek(start, SEEK_SET);
     f.readf(buf->data, frames);
-    return 0;
 }
 
 
-int sc_plugin_interface::buffer_alloc_read_channels(uint32_t index, const char * filename, uint32_t start,
-                                                    uint32_t frames, uint32_t channel_count,
-                                                    const uint32_t * channel_data)
+void sc_plugin_interface::buffer_alloc_read_channels(uint32_t index, const char * filename, uint32_t start,
+                                                     uint32_t frames, uint32_t channel_count,
+                                                     const uint32_t * channel_data)
 {
     SndfileHandle f(filename);
-    if (!f)
-        return -1; /* file cannot be opened */
+    if (f.rawHandle() == nullptr)
+        throw std::runtime_error(f.strError());
 
     uint32_t sf_channels = uint32_t(f.channels());
     const uint32_t * max_chan = std::max_element(channel_data, channel_data + channel_count);
     if (*max_chan >= sf_channels)
-        return -2;
+        throw std::runtime_error("Channel out of range");
 
     const size_t sf_frames = f.frames();
 
@@ -910,73 +948,6 @@ int sc_plugin_interface::buffer_alloc_read_channels(uint32_t index, const char *
 
     f.seek(start, SEEK_SET);
     read_channel(f, channel_count, channel_data, frames, buf->data);
-
-    return 0;
-}
-
-/* directly taken from supercollider sources
-   Copyright (c) 2002 James McCartney. All rights reserved.
-*/
-int sampleFormatFromString(const char* name)
-{
-    if (!name) return SF_FORMAT_PCM_16;
-
-    size_t len = strlen(name);
-    if (len < 1) return 0;
-
-    if (name[0] == 'u') {
-        if (len < 5) return 0;
-        if (name[4] == '8') return SF_FORMAT_PCM_U8; // uint8
-            return 0;
-    } else if (name[0] == 'i') {
-        if (len < 4) return 0;
-        if (name[3] == '8') return SF_FORMAT_PCM_S8;      // int8
-            else if (name[3] == '1') return SF_FORMAT_PCM_16; // int16
-                else if (name[3] == '2') return SF_FORMAT_PCM_24; // int24
-                    else if (name[3] == '3') return SF_FORMAT_PCM_32; // int32
-    } else if (name[0] == 'f') {
-        return SF_FORMAT_FLOAT; // float
-    } else if (name[0] == 'd') {
-        return SF_FORMAT_DOUBLE; // double
-    } else if (name[0] == 'm' || name[0] == 'u') {
-        return SF_FORMAT_ULAW; // mulaw ulaw
-    } else if (name[0] == 'a') {
-        return SF_FORMAT_ALAW; // alaw
-    }
-    return 0;
-}
-
-int headerFormatFromString(const char *name)
-{
-    if (!name) return SF_FORMAT_AIFF;
-    if (strcasecmp(name, "AIFF")==0) return SF_FORMAT_AIFF;
-    if (strcasecmp(name, "AIFC")==0) return SF_FORMAT_AIFF;
-    if (strcasecmp(name, "RIFF")==0) return SF_FORMAT_WAV;
-    if (strcasecmp(name, "WAVEX")==0) return SF_FORMAT_WAVEX;
-    if (strcasecmp(name, "WAVE")==0) return SF_FORMAT_WAV;
-    if (strcasecmp(name, "WAV" )==0) return SF_FORMAT_WAV;
-    if (strcasecmp(name, "Sun" )==0) return SF_FORMAT_AU;
-    if (strcasecmp(name, "IRCAM")==0) return SF_FORMAT_IRCAM;
-    if (strcasecmp(name, "NeXT")==0) return SF_FORMAT_AU;
-    if (strcasecmp(name, "raw")==0) return SF_FORMAT_RAW;
-    if (strcasecmp(name, "MAT4")==0) return SF_FORMAT_MAT4;
-    if (strcasecmp(name, "MAT5")==0) return SF_FORMAT_MAT5;
-    if (strcasecmp(name, "PAF")==0) return SF_FORMAT_PAF;
-    if (strcasecmp(name, "SVX")==0) return SF_FORMAT_SVX;
-    if (strcasecmp(name, "NIST")==0) return SF_FORMAT_NIST;
-    if (strcasecmp(name, "VOC")==0) return SF_FORMAT_VOC;
-    if (strcasecmp(name, "W64")==0) return SF_FORMAT_W64;
-    if (strcasecmp(name, "PVF")==0) return SF_FORMAT_PVF;
-    if (strcasecmp(name, "XI")==0) return SF_FORMAT_XI;
-    if (strcasecmp(name, "HTK")==0) return SF_FORMAT_HTK;
-    if (strcasecmp(name, "SDS")==0) return SF_FORMAT_SDS;
-    if (strcasecmp(name, "AVR")==0) return SF_FORMAT_AVR;
-    if (strcasecmp(name, "SD2")==0) return SF_FORMAT_SD2;
-    if (strcasecmp(name, "FLAC")==0) return SF_FORMAT_FLAC;
-    if (strcasecmp(name, "vorbis")==0) return SF_FORMAT_VORBIS;
-    if (strcasecmp(name, "CAF")==0) return SF_FORMAT_CAF;
-    if (strcasecmp(name, "RF64")==0) return SF_FORMAT_RF64;
-    return 0;
 }
 
 
@@ -1006,32 +977,30 @@ int sc_plugin_interface::buffer_write(uint32_t index, const char * filename, con
     return 0;
 }
 
-static int buffer_read_verify(SndfileHandle const & sf, size_t min_length, size_t samplerate)
+static void buffer_read_verify(SndfileHandle & sf, size_t min_length, size_t samplerate, bool check_samplerate)
 {
-    if (!sf)
-        return -1;
+    if (sf.rawHandle() == nullptr)
+        throw std::runtime_error(sf.strError());
     if (sf.frames() < min_length)
-        return -2; /* no more frames to read */
-    if (sf.samplerate() != samplerate)
-        return -3; /* sample rate mismatch */
-    return 0;
+        throw std::runtime_error("no more frames to read");
+
+    if (check_samplerate && (sf.samplerate() != samplerate))
+        throw std::runtime_error("sample rate mismatch");
 }
 
-int sc_plugin_interface::buffer_read(uint32_t index, const char * filename, uint32_t start_file, uint32_t frames,
-                                     uint32_t start_buffer, bool leave_open)
+void sc_plugin_interface::buffer_read(uint32_t index, const char * filename, uint32_t start_file, uint32_t frames,
+                                      uint32_t start_buffer, bool leave_open)
 {
     SndBuf * buf = World_GetNRTBuf(&world, index);
 
     if (uint32_t(buf->frames) < start_buffer)
-        return -2; /* buffer already full */
+        throw std::runtime_error("buffer already full");
 
     SndfileHandle sf(filename, SFM_READ);
-    int error = buffer_read_verify(sf, start_file, buf->samplerate);
-    if (error)
-        return error;
+    buffer_read_verify(sf, start_file, buf->samplerate, !leave_open);
 
     if (sf.channels() != buf->channels)
-        return -3; /* sample rate or channel count mismatch */
+        throw std::runtime_error("channel count mismatch");
 
     const uint32_t buffer_remain = buf->frames - start_buffer;
     const uint32_t file_remain = sf.frames() - start_file;
@@ -1042,41 +1011,38 @@ int sc_plugin_interface::buffer_read(uint32_t index, const char * filename, uint
 
     if (leave_open)
         buf->sndfile = sf.takeOwnership();
-    return 0;
 }
 
-int sc_plugin_interface::buffer_read_channel(uint32_t index, const char * filename, uint32_t start_file, uint32_t frames,
+void sc_plugin_interface::buffer_read_channel(uint32_t index, const char * filename, uint32_t start_file, uint32_t frames,
                                              uint32_t start_buffer, bool leave_open, uint32_t channel_count,
                                              const uint32_t * channel_data)
 {
     SndBuf * buf = World_GetNRTBuf(&world, index);
 
     if (channel_count != uint32_t(buf->channels))
-        return -2; /* channel count mismatch */
+        throw std::runtime_error("channel count mismatch");
 
     if (uint32_t(buf->frames) >= start_buffer)
-        return -2; /* buffer already full */
+        throw std::runtime_error("buffer already full");
 
     SndfileHandle sf(filename, SFM_READ);
-    int error = buffer_read_verify(sf, start_file, buf->samplerate);
-    if (error)
-        return error;
+    buffer_read_verify(sf, start_file, buf->samplerate, !leave_open);
 
     uint32_t sf_channels = uint32_t(sf.channels());
     const uint32_t * max_chan = std::max_element(channel_data, channel_data + channel_count);
     if (*max_chan >= sf_channels)
-        return -2;
+        throw std::runtime_error("channel count mismatch");
+
     const uint32_t buffer_remain = buf->frames - start_buffer;
     const uint32_t file_remain = sf.frames() - start_file;
 
     const uint32_t frames_to_read = std::min(frames, std::min(buffer_remain, file_remain));
 
     sf.seek(start_file, SEEK_SET);
-    read_channel(sf, channel_count, channel_data, frames, buf->data);
+    read_channel(sf, channel_count, channel_data, frames_to_read, buf->data);
 
     if (leave_open)
         buf->sndfile = sf.takeOwnership();
-    return 0;
 }
 
 void sc_plugin_interface::buffer_close(uint32_t index)
@@ -1108,7 +1074,7 @@ sample * sc_plugin_interface::buffer_generate(uint32_t buffer_index, const char*
     return sc_factory->run_bufgen(&world, cmd_name, buffer_index, &msg);
 }
 
-void sc_plugin_interface::buffer_sync(uint32_t index)
+void sc_plugin_interface::buffer_sync(uint32_t index) noexcept
 {
     sndbuf_copy(world.mSndBufs + index, world.mSndBufsNonRealTimeMirror + index);
     increment_write_updates(index);
